@@ -2,6 +2,7 @@ mod cash_flows;
 mod corporate_actions;
 mod dividends;
 mod fees;
+mod grants;
 mod interest;
 mod merging;
 mod partial;
@@ -48,6 +49,7 @@ pub use self::corporate_actions::{
     CorporateAction, CorporateActionType, StockSplitController, process_corporate_actions};
 pub use self::dividends::Dividend;
 pub use self::fees::Fee;
+pub use self::grants::{StockGrant, process_grants};
 pub use self::interest::IdleCashInterest;
 pub use self::merging::StatementsMergingStrategy;
 pub use self::reader::ReadingStrictness;
@@ -60,7 +62,7 @@ pub struct BrokerStatement {
     pub broker: BrokerInfo,
     pub period: Period,
 
-    pub cash_assets: MultiCurrencyCashAccount,
+    pub assets: NetAssets,
     pub historical_assets: BTreeMap<Date, NetAssets>,
 
     pub fees: Vec<Fee>,
@@ -75,6 +77,7 @@ pub struct BrokerStatement {
     pub stock_sells: Vec<StockSell>,
     pub dividends: Vec<Dividend>,
 
+    stock_grants: Vec<StockGrant>,
     corporate_actions: Vec<CorporateAction>,
     pub stock_splits: StockSplitController,
 
@@ -161,11 +164,12 @@ impl BrokerStatement {
             return Err!("Unable to find origin operations for the following taxes:\n{}{}", taxes, hint);
         }
 
+        process_grants(&mut statement, strictness.contains(ReadingStrictness::GRANTS))?;
+
         for (symbol, new_symbol) in symbol_remapping.iter() {
             statement.rename_symbol(symbol, new_symbol, None).map_err(|e| format!(
                 "Failed to remap {} to {}: {}", symbol, new_symbol, e))?;
         }
-
         statement.corporate_actions.extend(corporate_actions.iter().cloned());
 
         for (symbol, name) in instrument_names {
@@ -194,7 +198,7 @@ impl BrokerStatement {
         Ok(BrokerStatement {
             broker, period,
 
-            cash_assets: MultiCurrencyCashAccount::new(),
+            assets: NetAssets::default(),
             historical_assets: BTreeMap::new(),
 
             fees: Vec::new(),
@@ -209,6 +213,7 @@ impl BrokerStatement {
             stock_sells: Vec::new(),
             dividends: Vec::new(),
 
+            stock_grants: Vec::new(),
             corporate_actions: Vec::new(),
             stock_splits: StockSplitController::default(),
 
@@ -266,18 +271,26 @@ impl BrokerStatement {
         QuoteQuery::Stock(symbol.to_owned(), exchanges.get_prioritized())
     }
 
-    pub fn net_value(&self, converter: &CurrencyConverter, quotes: &Quotes, currency: &str) -> GenericResult<Cash> {
-        self.batch_quotes(quotes)?;
+    pub fn net_value(
+        &self, converter: &CurrencyConverter, quotes: &Quotes, currency: &str, realtime: bool,
+    ) -> GenericResult<Cash> {
+        let mut net_value = self.assets.cash.clone();
 
-        let mut net_value = self.cash_assets.total_assets_real_time(currency, converter)?;
+        match self.assets.other {
+            Some(other) if !realtime => {
+                net_value.deposit(other);
+            },
+            _ => {
+                self.batch_quotes(quotes)?;
 
-        for (symbol, quantity) in &self.open_positions {
-            let price = quotes.get(self.get_quote_query(symbol))?;
-            let price = converter.real_time_convert_to(price, currency)?;
-            net_value += quantity * price;
+                for (symbol, &quantity) in &self.open_positions {
+                    let price = quotes.get(self.get_quote_query(symbol))?;
+                    net_value.deposit(price * quantity);
+                }
+            },
         }
 
-        Ok(Cash::new(currency, net_value))
+        Ok(Cash::new(currency, net_value.total_assets_real_time(currency, converter)?))
     }
 
     pub fn emulate_sell(
@@ -323,8 +336,8 @@ impl BrokerStatement {
             return Err!("The portfolio has no open {} position", symbol);
         }
 
-        self.cash_assets.deposit(volume);
-        self.cash_assets.withdraw(commission);
+        self.assets.cash.deposit(volume);
+        self.assets.cash.withdraw(commission);
         self.stock_sells.push(stock_sell);
 
         Ok(())
@@ -335,7 +348,7 @@ impl BrokerStatement {
 
         for commissions in commission_calc.calculate()?.values() {
             for commission in commissions.iter() {
-                self.cash_assets.withdraw(commission);
+                self.assets.cash.withdraw(commission);
                 total.deposit(commission);
             }
         }
@@ -429,9 +442,8 @@ impl BrokerStatement {
         }
 
         if let partial::NetAssets{cash: Some(cash), other} = statement.assets {
-            self.cash_assets = cash.clone();
-
             let assets = NetAssets{cash, other};
+            self.assets = assets.clone();
             assert!(self.historical_assets.insert(self.period.last_date(), assets).is_none());
         } else if last {
             return Err!("Unable to find any information about current cash assets");
@@ -447,6 +459,7 @@ impl BrokerStatement {
         self.forex_trades.extend(statement.forex_trades.into_iter());
         self.stock_buys.extend(statement.stock_buys.into_iter());
         self.stock_sells.extend(statement.stock_sells.into_iter());
+        self.stock_grants.extend(statement.stock_grants.into_iter());
 
         self.corporate_actions.extend(statement.corporate_actions.into_iter());
         self.open_positions = statement.open_positions;
@@ -560,6 +573,7 @@ impl BrokerStatement {
 
         self.sort_and_validate_stock_buys()?;
         self.sort_and_validate_stock_sells()?;
+        validator.sort_and_validate("a stock grant", &mut self.stock_grants, |grant| grant.date)?;
 
         self.dividends.sort_by(|a, b| (a.date, &a.issuer).cmp(&(b.date, &b.original_issuer)));
         validator.validate("a dividend", &self.dividends, |dividend| dividend.date)?;
@@ -636,6 +650,7 @@ impl BrokerStatement {
     }
 }
 
+#[derive(Clone, Default)]
 pub struct NetAssets {
     pub cash: MultiCurrencyCashAccount,
     pub other: Option<Cash>, // Supported only for some brokers
