@@ -1,32 +1,31 @@
+pub mod alphavantage;
+mod cache;
+mod common;
+pub mod fcsapi;
+pub mod finnhub;
+mod moex;
+pub mod twelvedata;
+
 use std::cell::RefCell;
 use std::collections::{hash_map::Entry, HashMap};
 use std::sync::Arc;
 #[cfg(test)] use std::sync::Mutex;
 
-#[cfg(not(test))] use chrono::{DateTime, TimeZone};
 use itertools::Itertools;
-use lazy_static::lazy_static;
 use log::debug;
 use rayon::prelude::*;
-use regex::Regex;
 
 use crate::config::Config;
 use crate::core::{EmptyResult, GenericResult};
 use crate::currency::Cash;
 use crate::db;
 use crate::exchanges::{Exchange, Exchanges};
-#[cfg(not(test))] use crate::time;
 
 use self::cache::Cache;
+use self::common::parse_currency_pair;
+use self::fcsapi::FcsApi;
 use self::finnhub::Finnhub;
 use self::moex::Moex;
-use self::twelvedata::TwelveData;
-
-mod alphavantage;
-mod cache;
-mod finnhub;
-mod moex;
-mod twelvedata;
 
 #[derive(Clone)]
 pub enum QuoteQuery {
@@ -56,15 +55,15 @@ pub struct Quotes {
 
 impl Quotes {
     pub fn new(config: &Config, database: db::Connection) -> GenericResult<Quotes> {
-        let finnhub = config.finnhub.as_ref().ok_or(
-            "Finnhub configuration is not set in the configuration file")?;
+        let fcsapi = config.fcsapi.as_ref().ok_or(
+            "FCS API access key is not set in the configuration file")?;
 
-        let twelvedata = config.twelvedata.as_ref().ok_or(
-            "Twelve Data configuration is not set in the configuration file")?;
+        let finnhub = config.finnhub.as_ref().ok_or(
+            "Finnhub token is not set in the configuration file")?;
 
         Ok(Quotes::new_with(Cache::new(database, config.cache_expire_time, true), vec![
-            Arc::new(Finnhub::new(&finnhub.token)),
-            Arc::new(TwelveData::new(&twelvedata.token)),
+            Arc::new(Finnhub::new(finnhub)),
+            Arc::new(FcsApi::new(fcsapi)),
             Arc::new(Moex::new("TQTF")),
             Arc::new(Moex::new("TQBR")),
         ]))
@@ -292,25 +291,6 @@ pub fn get_currency_pair(base: &str, quote: &str) -> String {
     format!("{}/{}", base, quote)
 }
 
-fn parse_currency_pair(pair: &str) -> GenericResult<(&str, &str)> {
-    lazy_static! {
-        static ref REGEX: Regex = Regex::new(r"^(?P<base>[A-Z]{3})/(?P<quote>[A-Z]{3})$").unwrap();
-    }
-
-    let captures = REGEX.captures(pair).ok_or_else(|| format!(
-        "Invalid currency pair: {:?}", pair))?;
-
-    Ok((
-        captures.name("base").unwrap().as_str(),
-        captures.name("quote").unwrap().as_str(),
-    ))
-}
-
-#[cfg(not(test))]
-fn is_outdated_quote<T: TimeZone>(date_time: DateTime<T>) -> bool {
-    (time::utc_now() - date_time.naive_utc()).num_days() >= 5
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
